function [signals,PC,V] = pcasvd(data, n, compressiontheshold)
% author : Cheng Ren, Beihang University, April 2011
% pcasvd: Perform PCA using SVD.
% data - MxN matrix of input data
% (M dimensions, N trials)
% n - the number of dimension for the reduced data set
% compressiontheshold - optional, when this is set, n will be decided automatically, 
% reference paper: 'K. Fukunaga. Statistical Pattern Recognition, 2nd Edition. John Hopkins University Press, Baltimore, 1989.'
% signals - nxN matrix of projected data
% PC - each column is a PC
% V - Mx1 matrix of variances
[M,N] = size(data);
% subtract off the mean for each dimension
mn = mean(data,2);
data = data - repmat(mn,1,N);
% construct the matrix Y
Y = data' / sqrt(N-1);
% SVD does it all
[u,S,PC] = svd(Y);
% calculate the variances
S = diag(S);
V = S .* S;

if (nargin < 2)
    n = M;
end

if (nargin > 2)
    %
    sigmasqr = sum(V);

    sigmapartialsqr = 0;
    for i=1:M
        sigmapartialsqr = sigmapartialsqr + V(i);
        if(sigmapartialsqr >= compressiontheshold * sigmasqr)
            n = i;
            break;
        end
    end
end

% project the original data
signals = PC(:,1:n)' * data;





